<?php

namespace Drupal\compare_role_permissions\Form;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\PermissionHandlerInterface;
use Drupal\user\RoleStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Url;

/**
 * Class RolesCompareForm.
 *
 * @package Drupal\compare_role_permissions\Form
 */
class RolesCompareForm extends FormBase {

  /**
   * @var RequestStack
   */
  private $request;

  /**
   * Constructs a new UserPermissionsForm.
   *
   * @param \Drupal\user\PermissionHandlerInterface $permission_handler
   *   The permission handler.
   * @param \Drupal\user\RoleStorageInterface $role_storage
   *   The role storage.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(PermissionHandlerInterface $permission_handler, RoleStorageInterface $role_storage, ModuleHandlerInterface $module_handler, RequestStack $request
  ) {
    $this->permissionHandler = $permission_handler;
    $this->roleStorage = $role_storage;
    $this->moduleHandler = $module_handler;
    $this->request = $request;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('user.permissions'), $container->get('entity.manager')->getStorage('user_role'), $container->get('module_handler'), $container->get('request_stack')
    );
  }

  /**
   * Gets the roles to display in this form.
   *
   * @return \Drupal\user\RoleInterface[]
   *   An array of role objects.
   */
  protected function getRolePermission($role_name = '') {
    return $this->roleStorage->load($role_name)->getPermissions();
  }

  /**
   * Gets the roles to display in this form.
   *
   * @return \Drupal\user\RoleInterface[]
   *   An array of role objects.
   */
  protected function getRoleLabel($role_name = '') {
    return $this->roleStorage->load($role_name)->label();
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'roles_compare_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['select_role1'] = array(
      '#type' => 'select',
      '#title' => $this->t('Role 1'),
      '#options' => user_role_names(),
      '#required' => TRUE,
    );
    $form['select_role2'] = array(
      '#type' => 'select',
      '#required' => TRUE,
      '#title' => $this->t('Role 2'),
      '#options' => user_role_names(),
    );
    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Compare'),
      '#button_type' => 'primary',
    );
//    if ($form_state->isExecuted()) {
    $this->printRoleDiff($form, $form_state);
//    }
    return $form;
  }

  /**
   * @param array $form
   * @param FormStateInterface $form_state
   * @return array
   */
  public function printRoleDiff(array &$form, FormStateInterface $form_state) {
    if ($form_state->isExecuted()) {
      $doc_options['attributes'] = array('class' => 'permission-link');
      $role1 = $form_state->getValue('select_role1');
      $role2 = $form_state->getValue('select_role2');
      $role1_permissions = $this->getRolePermission($role1);
      $role2_permissions = $this->getRolePermission($role2);
      $role_diff = array_merge(
        array_diff($role2_permissions, $role1_permissions), array_diff($role1_permissions, $role2_permissions)
      );
      if (!empty($role_diff)) {
        $permissions = $this->permissionHandler->getPermissions();
        $listedPermissions = array();
        foreach ($permissions as $permission_name => $permission) {
          if (in_array($permission_name, $role_diff))
            $listedPermissions[$permission_name] = $permission;
        }
        $form['permissions'] = array(
          '#type' => 'table',
          '#header' => array(
            'module' => 'Module',
            'permission' => 'Permission',
            'role1' => $this->getRoleLabel($role1),
            'role2' => $this->getRoleLabel($role2),
          ),
        );
        foreach ($listedPermissions as $perm => $perm_item) {
          $form['permissions'][$perm]['module'] = array(
            '#markup' => $this->moduleHandler->getName($perm_item['provider'])
          );
          $form['permissions'][$perm]['permission'] = array(
            '#markup' => $perm_item['title']
          );
          foreach ([$role1, $role2] as $rid) {
            $form['permissions'][$perm][$rid] = array(
              '#title' => ': ' . $perm_item['title'],
              '#title_display' => 'invisible',
              '#wrapper_attributes' => array(
                'class' => array('checkbox'),
              ),
              '#type' => 'checkbox',
              '#default_value' => in_array($perm, $this->getRolePermission($rid)) ? 1 : 0,
              '#attributes' => array('class' => array('rid-' . $rid, 'js-rid-' . $rid)),
              '#parents' => array($rid, $perm),
            );
          }
        }
        $form['actions']['sync_perms'] = array(
          '#type' => 'submit',
          '#value' => $this->t('Update Permissions'),
          '#button_type' => 'primary',
        );
      } else {
        $form['no_diff'] = array(
          '#markup' => 'Both the selected roles has similar permissions',
        );
      }
      return $form;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Fetch the role 1.
    $role1 = $form_state->getValue('select_role1');
    // Fetch the role 2.
    $role2 = $form_state->getValue('select_role2');
    if ($role1 == $role2) {
      $form_state->setErrorByName('select_role2', $this->t('Role should be different from first.'));
    }
  }

  /**
   * @param array $form
   * @param FormStateInterface $form_state
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getTriggeringElement()['#value']->getUntranslatedString() == 'Update Permissions') {
      $roles = [$form_state->getValue('select_role1'), $form_state->getValue('select_role2')];
      foreach ($roles as $role_name) {
        user_role_change_permissions($role_name, (array)$form_state->getValue($role_name));
      }
      drupal_set_message($this->t('The changes have been saved.'));
    } else {
      $form_state->setRebuild();
    }
  }
}
